SuperStrict

Type TMandelBrotSet
	Field MaxIterations:Int		= 256
	Field DivergenceTreshold:Float	= 4
	
	Field RealMin:Float		= -4
	Field RealMax:Float		=  4
	Field ImaginaryMin:Float	= -3
	Field ImaginaryMax:Float	=  3
	
	Field ProgramObject:Int
	Field VertexShader:Int
	Field FragmentShader:Int
	
	Method Render( X:Int, Y:Int, Width:Int, Height:Int )
		glUseProgramObjectARB( ProgramObject )
		
		glUniform1iARB( glGetUniformLocationARB( ProgramObject, "MaxIterations" ), MaxIterations )
		glUniform1fARB( glGetUniformLocationARB( ProgramObject, "Treshold" ),      DivergenceTreshold )
		
		glBegin( GL_QUADS )
			glTexCoord2f( RealMin, ImaginaryMin ); glVertex2f( X,         Y          )
			glTexCoord2f( RealMax, ImaginaryMin ); glVertex2f( X + Width, Y          )
			glTexCoord2f( RealMax, ImaginaryMax ); glVertex2f( X + Width, Y + Height )
			glTexCoord2f( RealMin, ImaginaryMax ); glVertex2f( X,         Y + Height )
		glEnd()
		
		glUseProgramObjectARB( 0 )
	End Method
	
	Method InitShader()
		ProgramObject	= glCreateProgramObjectARB()
		VertexShader	= glCreateShaderObjectARB( GL_VERTEX_SHADER_ARB )
		FragmentShader	= glCreateShaderObjectARB( GL_FRAGMENT_SHADER_ARB )
		
		LoadShader( "", VertexShader,   GetVertexShader() )
		LoadShader( "", FragmentShader, GetFragmentShader() )
		
		glCompileShaderARB( VertexShader );   CheckForErrors( VertexShader )
		glCompileShaderARB( FragmentShader ); CheckForErrors( FragmentShader )
		
		glAttachObjectARB( ProgramObject, VertexShader )
		glAttachObjectARB( ProgramObject, FragmentShader )
		
		glDeleteObjectARB( VertexShader )
		glDeleteObjectARB( FragmentShader )
		
		glLinkProgramARB( ProgramObject ); CheckForErrors( ProgramObject )
	End Method
	
	Method SetZoomRect( CenterX:Int, CenterY:Int, Width:Int, Height:Int, ScreenWidth:Int, ScreenHeight:Int )
		Local SpanX:Float = RealMax - RealMin
		Local SpanY:Float = ImaginaryMax - ImaginaryMin
		Local OldMinR:Float = RealMin
		Local OldMinI:Float = ImaginaryMin
		
		RealMin = RealMin + ( CenterX - Width/2 )*SpanX/Float( ScreenWidth )
		RealMax = OldMinR + ( CenterX + Width/2 )*SpanX/Float( ScreenWidth )
		ImaginaryMin = ImaginaryMin + ( CenterY - Height/2 )*SpanY/Float( ScreenHeight )
		ImaginaryMax = OldMinI      + ( CenterY + Height/2 )*SpanY/Float( ScreenHeight )
	End Method
	
	Function GetVertexShader:String()
		Return ..
		"void main( void ) {~n" + ..
			"gl_Position = gl_ModelViewProjectionMatrix*gl_Vertex;~n" + ..
			"gl_TexCoord[0]  = gl_MultiTexCoord0;~n" + ..
		"}"
	End Function
	
	Function GetFragmentShader:String()
		Return ..
		"uniform int MaxIterations;~n" + ..
		"uniform float Treshold;~n" + ..
		"~n" + ..
		"void main( void ) {~n" + ..
			"vec4 C = gl_TexCoord[ 0 ];~n" + ..
			"vec4 Z = C;~n" + ..
			"vec2 CSq;~n" + ..
			"~n" + ..
			"int Iteration = 0;~n" + ..
			"CSq.x = C.x*C.x;~n" + ..
			"CSq.y = C.y*C.y;~n" + ..
			"~n" + ..
			"while( Iteration++ < MaxIterations && CSq.x + CSq.y < Treshold ) {~n" + ..
				"Z.w = CSq.x - CSq.y + C.x;~n" + ..
				"Z.y = 2.0*Z.x*Z.y + C.y;~n" + ..
				"Z.x = Z.w;~n" + ..
				"CSq.x = Z.x*Z.x;~n" + ..
				"CSq.y = Z.y*Z.y;~n" + ..
			"}~n" + ..
			"~n" + ..
			"float Ratio = float( Iteration )/float( MaxIterations )*7.0;~n" + ..
			"float Rise = mod( Ratio, 1.0 );~n" + ..
			"float Fall = 1.0 - Rise;~n" + ..
			"Ratio = floor( Ratio );~n" + ..
			"~n" + ..
			"     if( Ratio == 0.0 ) gl_FragColor = vec4( Rise, 0.0, 0.0, 1.0 );~n" + ..
			"else if( Ratio == 1.0 ) gl_FragColor = vec4( 1.0, Rise, 0.0, 1.0 );~n" + ..
			"else if( Ratio == 2.0 ) gl_FragColor = vec4( Fall, 1.0, 0.0, 1.0 );~n" + ..
			"else if( Ratio == 3.0 ) gl_FragColor = vec4( 0.0, 1.0, Rise, 1.0 );~n" + ..
			"else if( Ratio == 4.0 ) gl_FragColor = vec4( 0.0, Fall, 1.0, 1.0 );~n" + ..
			"else if( Ratio == 5.0 ) gl_FragColor = vec4( Rise, 0.0, 1.0, 1.0 );~n" + ..
			"else if( Ratio == 6.0 ) gl_FragColor = vec4( 1.0, Rise, 1.0, 1.0 );~n" + ..
		"}"
	End Function
End Type

Const GWIDTH:Int  = 800
Const GHEIGHT:Int = 600
Const ZOOM_WIDTH:Int  = 400
Const ZOOM_HEIGHT:Int = 300

InitGL()
CheckCompability()

Local Mandelbrot:TMandelBrotSet = New TMandelBrotSet
	Mandelbrot.InitShader()

Local Timer:TTimer = CreateTimer( 60 )

While Not ( KeyHit( KEY_ESCAPE ) Or AppTerminate() )
	glClear( GL_COLOR_BUFFER_BIT )
	
	Mandelbrot.Render( 0, 0, GWIDTH, GHEIGHT )
	
	If MouseHit( 1 ) Then Mandelbrot.SetZoomRect( MouseX(), MouseY(), ZOOM_WIDTH,   ZOOM_HEIGHT,   GWIDTH, GHEIGHT )
	If MouseHit( 2 ) Then Mandelbrot.SetZoomRect( MouseX(), MouseY(), ZOOM_WIDTH*4, ZOOM_HEIGHT*4, GWIDTH, GHEIGHT )
	
	Flip 0
	WaitTimer Timer
Wend
End

Function InitGL()
	GLGraphics 800, 600
	
	glewInit()
	
	glMatrixMode( GL_PROJECTION )
	glLoadIdentity()
	
	glOrtho( 0, GWIDTH, GHEIGHT, 0, -1, 1 )
	
	glMatrixMode( GL_MODELVIEW )
	glLoadIdentity()
End Function

Function CheckCompability()
	Local Extensions:String	= String.FromCString( Byte Ptr glGetString( GL_EXTENSIONS ) )
	Local GLVersion:String		= String.FromCString( Byte Ptr glGetString( GL_VERSION ) )
	Local GLVersionInt:Int		= GLVersion[ .. 3 ].Replace( ".", "" ).ToInt()
	
	If Extensions.Find( "GL_ARB_shader_objects"  ) >= 0 And ..
	   Extensions.Find( "GL_ARB_vertex_shader"   ) >= 0 And ..
	   Extensions.Find( "GL_ARB_fragment_shader" ) >= 0 Or GLVersionInt >= 20 Then Return
	
	Notify "GLSL wird nicht untersttzt.", True
	
	End
End Function

Function LoadShader( Path:String, ShaderObject:Int, ExistingShaderCode:String = "" )
	Local ShaderCode:String
	
	If ExistingShaderCode <> "" Then
		ShaderCode = ExistingShaderCode
	Else
		Try ShaderCode = LoadText( Path ) Catch Error:Object Return; EndTry
	EndIf
	
	If ShaderCode <> "" Then
		Local ShaderCodeC:Byte Ptr		= ShaderCode.ToCString()
		Local ShaderCodeLen:Int		= ShaderCode.Length
		
		glShaderSourceARB( ShaderObject, 1, Varptr ShaderCodeC, Varptr ShaderCodeLen )
		
		MemFree( ShaderCodeC )
	EndIf
End Function

Function CheckForErrors( ShaderObject:Int )
	Local ErrorLength:Int
	
	glGetObjectParameterivARB( ShaderObject, GL_OBJECT_INFO_LOG_LENGTH_ARB, Varptr ErrorLength )
	
	If ErrorLength Then
		Local Message:Byte[ ErrorLength ], Dummy:Int
		
		glGetInfoLogARB( ShaderObject, ErrorLength, Varptr Dummy, Varptr Message[ 0 ] )
		
		WriteStdout "Shader object '" + ShaderObject + "': " + StringFromCharArray( Message )
		WriteStdout "~n"
	EndIf
End Function

Function StringFromCharArray:String( Array:Byte[] )
	Local Output:String
	For Local I:Int = 0 To Array.Length - 1
		Output :+ Chr( Array[ I ] )
	Next
	
	Return Output
End Function